/*
 * This file is licensed under the Elastic License 2.0. Copyright 2021-present, StarRocks Inc.
 */

{% macro starrocks__olap_table(is_create_table_as) -%}

  {% set is_create_table = is_create_table_as is none or not is_create_table_as %}

  {% set table_type = config.get('table_type', 'DUPLICATE') %}
  {% set keys = config.get('keys') %}
  {% set partition_by = config.get('partition_by') %}
  {% set partition_by_init = config.get('partition_by_init') %}
  {% set buckets = config.get('buckets', 10) %}
  {% set distributed_by = config.get('distributed_by') %}
  {% if is_create_table %}
    {% set properties = config.get('properties', {"replication_num":"1"}) %}
  {% else %}
    {% set properties = config.get('properties') %}
  {% endif %}

  {# 1. SET ENGINE #}
  {% if is_create_table %}
    ENGINE = OLAP
  {% endif %}

  {# 2. SET KEYS #}
  {% if is_create_table %}
    {% if keys is not none %}
      {% if table_type == "DUPLICATE" %}
        DUPLICATE KEY (
          {% for item in keys %}
            {{ item }}{% if not loop.last %},{% endif %}
          {% endfor %}
        )
      {% elif table_type == "PRIMARY" %}
        PRIMARY KEY (
          {% for item in keys %}
            {{ item }}{% if not loop.last %},{% endif %}
          {% endfor %}
        )
      {% elif table_type == "UNIQUE" %}
        UNIQUE KEY (
          {% for item in keys %}
            {{ item }}{% if not loop.last %},{% endif %}
          {% endfor %}
        )
      {% else %}
        {% set msg -%}
          "{{ table_type }}" is not support
        {%- endset %}
        {{ exceptions.raise_compiler_error(msg) }}
      {% endif %}
    {% else %}
      {% if table_type != "DUPLICATE" %}
        {% set msg -%}
          "{{ table_type }}" is must set "keys"
        {%- endset %}
        {{ exceptions.raise_compiler_error(msg) }}
      {% endif %}
    {% endif %}
  {% endif %}

  {# 3. SET PARTITION #}
  {{ starrocks__partition_by(partition_by, partition_by_init) }}

  {# 4. SET DISTRIBUTED #}
  {% if distributed_by is not none %}
    DISTRIBUTED BY HASH (
      {% for item in distributed_by %}
        {{ item }}{% if not loop.last %},{% endif %}
      {% endfor %}
    ) BUCKETS {{ buckets }}
  {% else %}
    {% set msg -%}
      [distributed_by] is not set
    {%- endset %}
    {{ exceptions.raise_compiler_error(msg) }}
  {% endif %}

  {# 4. SET PROPERTIES #}
  {% if properties is not none %}
    PROPERTIES (
      {% for key, value in properties.items() %}
        "{{ key }}" = "{{ value }}"{% if not loop.last %},{% endif %}
      {% endfor %}
    )
  {% endif %}
{%- endmacro %}

{% macro starrocks__other_table() -%}
  {% set engine = config.get('engine') %}
  {% set properties = config.get('properties') %}

  ENGINE = {{ engine }}
  {% if properties is not none %}
    PROPERTIES (
      {% for key, value in properties.items() %}
        "{{ key }}" = "{{ value }}"{% if not loop.last %},{% endif %}
      {% endfor %}
    )
  {% endif %}
{%- endmacro %}

{% macro starrocks__partition_by(cols, init) -%}
  {% if cols is not none %}
    PARTITION BY RANGE (
      {% for col in cols %}
        {{ col }}{% if not loop.last %},{% endif %}
      {% endfor %}
    )(
      {% if init is not none %}
        {% for row in init %}
          {{ row }}{% if not loop.last %},{% endif %}
        {% endfor %}
      {% endif %}
    )
  {% endif %}
{%- endmacro %}